# Copyright 2018 Ruben Quinones (ruben.quinones@rackspace.com)
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from mercury.boot.views import BootView, DiscoverView

boot_view = BootView.as_view('boot')
discover_view = DiscoverView.as_view('discover')

boot_urls = [
    # Boot url rules
    ('/boot', boot_view),
    # Discover rules
    ('/boot/discover/<mac_address>', discover_view),
]
